;------------------------------------------------------------------------------
; Fu Kung! Copyright (C)2003 Andrew Davie
; see http://www.taswegian.com/TwoHeaded/Atari2600/fukung.html << SOON


; The genesis of the large-sprite system contained herein comes from Eckhard Stohlberg's
; fine work on the 'BIGTWO' sprite system (see the [stella] list archives at
; http://www.biglist.com/lists/stella/archives/ ) and in his preliminary fighter code
; kindly loaned to assist in creating this program.

; The sprite system this program uses is somewhat different to Eckhard's - being more
; efficient and capable (in terms of sprite display) but does essentially retain the
; positioning and movement system of the initial system.  Many thanks ES!

; This code is for a 512KB cartridge using the $3F bank-switching scheme.  If running
; with Z26 emulator (v1.57+) use -g11 switch on command-line.

;------------------------------------------------------------------------------

                processor 6502
                include "vcs.h"
                include "macro.h"

BGCHRONO        = 0

DEBUG           = 0         ; non-zero for debug stuff

BGCOL           = 0
PFCOL           = 0 ;%011100100 

D0              = 1
D1              = 2
D2              = 4
D3              = 8
D4              = 16
D5              = 32
D6              = 64
D7              = 128

BANK            = $3F       ; write address to switch banks
BANKPAGE        = $F000     ; base address of bank blocks

;------------------------------------------------------------------------------
; Code automatically compiles to implement the sprites in the size/format
; specified here.  Can handle up to 6 rows.  

;------------------------------------------------------------------------------
; COLOUR BITMAP COLOUR CYCLING TESTBED
; Code Copyright (C)2003 Andrew Davie -- atari2600@taswegian.com
; May be freely used for private, educational and research purposes.
; If you wish to profit from it, please ask me for permission first.
; Distributed to the [stella] list 30/March/2003


DIRECTION = 1           ; 0 = inwards,  non-0 = outwards

BIG_ROWS        = 1
BIG_COLS        = 6         ; DO NOT CHANGE

BIG_SPRITES     = 1         ; #creatures

;------------------------------------------------------------------------------
; MACRO definitions 

            MAC NEWBANK ; bank name
            SEG {1}
            ORG ORIGIN
            RORG $F000
{1}         SET ORIGIN / 2048
ORIGIN      SET ORIGIN + 2048
            ENDM

    ;--------------------------------------------------------------------------


    MAC VECTOR              ; just a word pointer to code
        .word {1}
    ENDM


    MAC SETBANK
            lda #{1}
            sta RAM_BANK
            sta BANK
    ENDM


    MAC ANIMATION
            lda #<{1}
            sta ObjAnimLO,x
            lda #>{1}
            sta ObjAnimHI,x
            lda #1
            sta ObjDelay,x

			lda ObjFlag,x
			ora #OBJFLAG_WAIT			; animation in progress
			sta ObjFlag,x

    ENDM

    MAC HANDLER
            ldy #{1}*2
            jsr SetHandler
    ENDM

    MAC STATE       ; does not continue after macro!

            ldy #STATE_{1}*2
            jmp SetState

    ENDM


;------------------------------------------------------------------------------

                SEG.U variables
                ORG $80

DrawX           ds 2
DrawY           ds 1
CenterX         ds 2
CenterY         ds 1
FrameBank       ds 1

VisualX         ds 2
VisualY         ds 1

LoopCount       ds 1
SPBASE          ds BIG_ROWS * BIG_COLS * 2

TPlayer         ds 1

BGColour        ds 1
BGPlayer        ds 1

Temp            ds 2
Anim            ds 2
RGBVec          ds 2
FrameCount      ds 1

Player          ds 1
FrameAddress    ds 2
Phase           ds 1
Phase2          ds 1
switch          ds 1

RAM_BANK        ds 1
PCount          ds 1
Audio           ds 2

;----
Flags           ds 1

                ; -------L
                ; L = locked to opponent

FLAG_LOCK       = D0

LineR0          ds 3 * 14            ; 6 columns of each of RED, GREEN, BLUE then 3 audios


;----
; Object variables


ObjXPosition    ds BIG_SPRITES
ObjYPosition    ds BIG_SPRITES

ObjFacing         ds BIG_SPRITES

                ; F-------
                ; F = facing direction (0 = right, 1 = left)
                ; - = unused

ObjDelay        ds BIG_SPRITES              ; delay count for frame animation
ObjAnimLO       ds BIG_SPRITES              ; animation pointer lo byte
ObjAnimHI       ds BIG_SPRITES              ; animation pointer hi byte
objAnimBank     ds BIG_SPRITES              ; bank containing animation

    ; Note: Possibly include bank with animation hi byte

ObjStateLO      ds BIG_SPRITES              ; state table lo byte
ObjStateHI      ds BIG_SPRITES              ; state table hi byte

ObjFSMLO        ds BIG_SPRITES              ; FSM system lo byte
ObjFSMHI        ds BIG_SPRITES              ; FSM system hi byte

ObjFlag			ds BIG_SPRITES
ObjFrame        ds BIG_SPRITES
ObjPhase        ds BIG_SPRITES




Colour          ds 3
Count           ds 1

OBJFLAG_WAIT	= D7						; wait for animation


;------------------------------------------------------------------------------


OVERLAY_SIZE    SET 10
    MAC VALIDATE_OVERLAY
        #if * - Overlay > OVERLAY_SIZE
            ERR
        #endif
    ENDM



        ; This overlay variable is used for the overlay variables.  That's OK.
        ; However, it is positioned at the END of the variables so, if on the off chance we're overlapping
        ; stack space and variable, it is LIKELY that that won't be a problem, as the temp variables
        ; (especially the latter ones) are only used in rare occasions.

        ; FOR SAFETY, DO NOT USE THIS AREA DIRECTLY (ie: NEVER reference 'Overlay' in the code)
        ; ADD AN OVERLAY FOR EACH ROUTINE'S USE, SO CLASHES CAN BE EASILY CHECKED

Overlay         ds OVERLAY_SIZE       ;--> overlay (share) variables
                VALIDATE_OVERLAY

;------------------------------------------------------------------------------
; OVERLAYS!
; These variables are overlays, and should be managed with care
; That is, variables are ALREADY DEFINED, and we're reusing RAM for other purposes

; EACH OF THESE ARE VARIABLES (TEMPORARY) USED BY ONE ROUTINE (AND IT'S SUBROUTINES)
; THAT IS, LOCAL VARIABLES.  USE 'EM FREELY, THEY COST NOTHING

; TOTAL SPACE USED BY ANY OVERLAY GROUP SHOULD BE <= SIZE OF 'Overlay'

;------------------------------------------------------------------------------

                SEG.U OverlayX
                org Overlay
                VALIDATE_OVERLAY

;------------------------------------------------------------------------------

                SEG.U ChronoOverlay
                org Overlay
ChronoBank      ds 3
ChronoColour    ds 3
ScreenPtr       ds 2
                VALIDATE_OVERLAY

;==============================================================================
; Include all the graphics into separate 2K segments

ORIGIN      SET $00000

            NEWBANK FRAMEBANK09
            include "Graphics\bank09.asm"

            NEWBANK FRAMEBANK02
            include "Graphics\bank02.asm"

            NEWBANK FRAMEBANK03
            include "Graphics\bank03.asm"
            
            NEWBANK FRAMEBANK11
            include "Graphics\bank11.asm"
            
            NEWBANK FRAMEBANK12
            include "Graphics\bank12.asm"
            
            NEWBANK FRAMEBANK10
            include "Graphics\bank10.asm"

            NEWBANK FRAMEBANK06
            include "Graphics\bank06.asm"

            NEWBANK FRAMEBANK07
            include "Graphics\bank07.asm"

            NEWBANK FRAMEBANK08
            include "Graphics\bank08.asm"

            NEWBANK FRAMEBANK13
            include "Graphics\bank13.asm"

            NEWBANK FRAMEBANK00
            include "Graphics\bank00.asm"

            NEWBANK FRAMEBANK01
            include "Graphics\bank01.asm"

            NEWBANK FRAMEBANK04
            include "Graphics\bank04.asm"

            NEWBANK FRAMEBANK05
            include "Graphics\bank05.asm"

;==============================================================================

            NEWBANK BLUEBANK
            include "testblue.asm"

;------------------------------------------------------------------------------
            NEWBANK REDBANK
            include "testred.asm"

;------------------------------------------------------------------------------
            NEWBANK GREENBANK
            include "testgreen.asm"


            NEWBANK MOVIE0
            include "Movie\0red.asm"

            NEWBANK MOVIE1
            include "Movie\0blue.asm"

            NEWBANK MOVIE2
            include "Movie\1green.asm"

            NEWBANK MOVIE3
            include "Movie\2red.asm"
            include "Movie\2green.asm"

            NEWBANK MOVIE4
            include "Movie\2blue.asm"
            include "Movie\3red.asm"

            NEWBANK MOVIE5
            include "Movie\3green.asm"
            include "Movie\3blue.asm"

            NEWBANK MOVIE6
            include "Movie\4red.asm"
            include "Movie\4green.asm"

            NEWBANK MOVIE7
            include "Movie\4blue.asm"
            include "Movie\5red.asm"

            NEWBANK MOVIE8
            include "Movie\5green.asm"
            include "Movie\5blue.asm"

            NEWBANK MOVIE0B
            include "Movie\0green.asm"

            NEWBANK MOVIE1B
            include "Movie\1red.asm"

            NEWBANK MOVIE1BLUE
            include "Movie\1blue.asm"


;------------------------------------------------------------------------------
            NEWBANK TITLESCREEN



    ; The image file is composed of strips (corresponding to PF0-PF2, repeated)
    ; which are decomposed by a utility from colour-separated RGB components of
    ; an original image. The displayed image is 40 pixels x 192 pixels.

    ; For access to the conversion/decomposition tool which generated these data
    ; please contact me privately.

;------------------------------------------------------------------------------


TitleScreen

            sty FrameCount

                ; Calculate address data for requested screen

            asl
            tay                 ; word index
            lda ChronoScreen,y
            sta ScreenPtr
            lda ChronoScreen+1,y
            sta ScreenPtr+1


            ldy #0

            lda (ScreenPtr),y
            sta ChronoBank
            
            iny
            lda (ScreenPtr),y
            sta ChronoBank+1

            iny
            lda (ScreenPtr),y
            sta ChronoBank+2

            

            ldx #0

SetupLinePtr
            iny
            lda (ScreenPtr),y
            sta LineR0,x
            inx
            cpx #42
            bne SetupLinePtr



    ;---------------------------------------------------------------------------

            lda #RED
            sta Colour
            lda #GREEN
            sta Colour+1
            lda #BLUE
            sta Colour+2


          

            lda #0
            sta Phase

AnotherFrame

            jsr TitleFrame
            jsr TitleFrame
            jsr TitleFrame

            dec FrameCount
            bne AnotherFrame

            rts





TitleFrame


    ; Start of vertical blank processing


NewFrame2    lda #2
            sta WSYNC

            sta VBLANK          

            sta VSYNC           
            sta WSYNC           
            sta WSYNC
            sta WSYNC

            lda #0
            sta VSYNC           

            lda #43             
            sta TIM64T

            lda #0
            sta COLUBK

VblankLoop2  lda INTIM
            bne VblankLoop2      
            
             sta WSYNC           
            sta VBLANK          

    ;------------------------------------------------------------------------------
    ; START OF DISPLAY


            lda #238                ; NTSC
            sta TIM64T

            lda #%00010100      ; reflect, priority, 2 pixel wide ball
            sta CTRLPF
          
            lda #0
            sta PF0
            sta PF1
            sta PF2

    ;--------------------------------------------------------------------------
    ; START OF COLOUR BITMAP DISPLAY KERNEL

            ldx Phase
            lda NextPhase,x
            sta Phase

#if 0
    ; Now cycle the colours

            inc Count
            lda Count
            cmp #12                 ; this controls cycling speed - change as you wish
            bcc Nzap

    ; fade code
            lda Colour
            pha
            and #$F0
            sta Colour
            pla
            sec
            sbc #1
            and #%1111
            ora Colour
            sta Colour

            lda Colour+1
            pha
            and #$F0
            sta Colour+1
            pla
            sec
            sbc #1
            and #%1111
            ora Colour+1
            sta Colour+1

            lda Colour+2
            pha
            and #$F0
            sta Colour+2
            pla
            sec
            sbc #1
            and #%1111
            ora Colour+2
            sta Colour+2
#endif



;    #if DIRECTION = 0   ;inwards

;            ldx Colour
;            lda Colour+2
;            sta Colour
;            lda Colour+1
;            sta Colour+2
;            stx Colour+1

;    #else
;            ldx Colour
;            lda Colour+1
;            sta Colour
;            lda Colour+2
;            sta Colour+1
;            stx Colour+2

;    #endif


            lda #0
            sta Count

Nzap




            lda #RED
            sta ChronoColour
            lda #GREEN
            sta ChronoColour+1
            lda #BLUE
            sta ChronoColour+2

            jsr ChronoDraw

         

    ; Screen has been drawn - now just tidy up

            sty WSYNC
            sty PF0
            sty PF1
            sty PF2

PadScreen2b  lda INTIM
            bne PadScreen2b
  
            sta WSYNC

            lda #%01000010
            sta VBLANK                      ; end of screen - enter blanking

            lda #84 ;25
            sta TIM64T

Overscan2    lda INTIM
            bne Overscan2

            sta WSYNC           
            sta WSYNC           

            rts



ChronoScreen


    .word BeachFrame0
    .word BeachFrame1
    .word BeachFrame2
    .word BeachFrame3
    .word BeachFrame4
    .word BeachFrame5


BeachFrame0
    .byte MOVIE0, MOVIE0B, MOVIE1
    .word testred0_STRIP_0, testred0_STRIP_1, testred0_STRIP_2, testred0_STRIP_3, testred0_STRIP_4, testred0_STRIP_5, AUDIO0RED
    .word testgreen0_STRIP_0, testgreen0_STRIP_1, testgreen0_STRIP_2, testgreen0_STRIP_3, testgreen0_STRIP_4, testgreen0_STRIP_5, $8000
    .word testblue0_STRIP_0, testblue0_STRIP_1, testblue0_STRIP_2, testblue0_STRIP_3, testblue0_STRIP_4, testblue0_STRIP_5, $8000

BeachFrame1
    .byte MOVIE1B, MOVIE2, MOVIE1BLUE
    .word testred1_STRIP_0, testred1_STRIP_1, testred1_STRIP_2, testred1_STRIP_3, testred1_STRIP_4, testred1_STRIP_5, $8000
    .word testgreen1_STRIP_0, testgreen1_STRIP_1, testgreen1_STRIP_2, testgreen1_STRIP_3, testgreen1_STRIP_4, testgreen1_STRIP_5, $8000
    .word testblue1_STRIP_0, testblue1_STRIP_1, testblue1_STRIP_2, testblue1_STRIP_3, testblue1_STRIP_4, testblue1_STRIP_5, $8000

BeachFrame2
    .byte MOVIE3, MOVIE3, MOVIE4
    .word testred2_STRIP_0, testred2_STRIP_1, testred2_STRIP_2, testred2_STRIP_3, testred2_STRIP_4, testred2_STRIP_5, $8000
    .word testgreen2_STRIP_0, testgreen2_STRIP_1, testgreen2_STRIP_2, testgreen2_STRIP_3, testgreen2_STRIP_4, testgreen2_STRIP_5, $8000
    .word testblue2_STRIP_0, testblue2_STRIP_1, testblue2_STRIP_2, testblue2_STRIP_3, testblue2_STRIP_4, testblue2_STRIP_5, $8000

BeachFrame3
    .byte MOVIE4, MOVIE5, MOVIE5
    .word testred3_STRIP_0, testred3_STRIP_1, testred3_STRIP_2, testred3_STRIP_3, testred3_STRIP_4, testred3_STRIP_5, $8000
    .word testgreen3_STRIP_0, testgreen3_STRIP_1, testgreen3_STRIP_2, testgreen3_STRIP_3, testgreen3_STRIP_4, testgreen3_STRIP_5, $8000
    .word testblue3_STRIP_0, testblue3_STRIP_1, testblue3_STRIP_2, testblue3_STRIP_3, testblue3_STRIP_4, testblue3_STRIP_5, $8000

BeachFrame4
    .byte MOVIE6, MOVIE6, MOVIE7
    .word testred4_STRIP_0, testred4_STRIP_1, testred4_STRIP_2, testred4_STRIP_3, testred4_STRIP_4, testred4_STRIP_5, $8000
    .word testgreen4_STRIP_0, testgreen4_STRIP_1, testgreen4_STRIP_2, testgreen4_STRIP_3, testgreen4_STRIP_4, testgreen4_STRIP_5, $8000
    .word testblue4_STRIP_0, testblue4_STRIP_1, testblue4_STRIP_2, testblue4_STRIP_3, testblue4_STRIP_4, testblue4_STRIP_5, $8000

BeachFrame5
    .byte MOVIE7, MOVIE7, MOVIE8
    .word testred5_STRIP_0, testred5_STRIP_1, testred5_STRIP_2, testred5_STRIP_3, testred5_STRIP_4, testred5_STRIP_5, $8000
    .word testgreen5_STRIP_0, testgreen5_STRIP_1, testgreen5_STRIP_2, testgreen5_STRIP_3, testgreen5_STRIP_4, testgreen5_STRIP_5, $8000
    .word testblue5_STRIP_0, testblue5_STRIP_1, testblue5_STRIP_2, testblue5_STRIP_3, testblue5_STRIP_4, testblue5_STRIP_5, $8000

;===============================================================================

            NEWBANK INITBANK

InitX
.XP     SET 80-48


    REPEAT BIG_SPRITES
    .byte .XP
.XP     SET .XP + 48
    REPEND


    ; There's no particular reason why this code lives in a separate bank
    ; but it's as good-a-test as any for checking the bankswitching is working


Cart_Init

            CLEAN_START
            
             
            lda #0
            sta SWBCNT          ; console I/O always set to INPUT
            sta SWACNT          ; set controller I/O to INPUT





GameInit

            lda #0
            sta Flags
            sta Player

    ; Initialise creatures

            ldx #BIG_SPRITES-1
PreClear    

            lda #0
            sta ObjFacing,x
			sta ObjFlag,x
            sta ObjPhase,x                  ; RGB 0/1/2 kernel display pase
;            sta ObjAnimHI,x
            
            lda #140
            sta ObjYPosition,x

            lda InitX,x
            sta ObjXPosition,x

            SETBANK INITBANK
            
            ldy #STATE_NORMAL*2
            jsr SetState                    ; also calls processFSM

;            SETBANK INITBANK
            jsr AnimateObject               ; process object animation sequence

            dex
            bpl PreClear

;loopx       inx
;            stx COLUBK
;            jmp loopx


    ;----------------------------------
    ; Hardwire init a single creature...

    #if BIG_SPRITES > 1

;            lda #D7
;            sta ObjFacing           ;todo currently fucked

            ldx #1
            HANDLER EVENT_INIT2

;            ldx #0
;            HANDLER 0

    #endif

;            ldx #0
;            HANDLER NORMAL_STAND
  

    ;----------------------------------


            lda #BGCOL
            sta BGColour
            jmp NewFrame



    include "Graphics\bank_FRAMETABLE.asm"


;------------------------------------------------------------------------------
; OBJECT Animation
; *MUST* BE IN ONE BANK


    MAC SHOW
            .byte {1},{2}
    ENDM

    MAC SHOWRGB
    REPEAT {2}
        SHOW FRAME_{1}RED,1
        ;SHOW FRAME_{1}GREEN,1
        SHOW FRAME_{1}BLUE,1
    REPEND
    ENDM

    MAC SYNCH
            .byte TOKEN_SYNCH
    ENDM
    
    MAC SETCOL
            .byte TOKEN_COLOUR
            .byte {1}
    ENDM

    MAC LOCK

            .byte TOKEN_LOCK
            .byte {1}*2

    ENDM


    MAC UNLOCK
            .byte TOKEN_UNLOCK
    ENDM

    MAC GOTO
            .byte TOKEN_JUMP
            .word {1}
    ENDM

	MAC DONE
			.byte TOKEN_DONE
	ENDM

    MAC MOVE
            .byte TOKEN_MOVE
            .word {1}
            .byte {2}
    ENDM

	MAC FLIP
			.byte TOKEN_FLIP
	ENDM



NOFRAME     equ $FF


noAnim      lda #NOFRAME
            sta ObjFrame,x
            rts


exitAnim    rts


AnimateObject


            lda ObjAnimHI,x
            beq noAnim                      ; animation switched off - use a null frame
            lda ObjAnimHI,x

            lda ObjDelay,x
            beq exitAnim                    ; static frame - no animation
            dec ObjDelay,x
            bne exitAnim                    ; not ready to change

ReAnimate   lda ObjAnimHI,x
            sta Anim+1
            lda ObjAnimLO,x
            sta Anim                        ; (Anim) points to frame sequence

ReReAnim    ldy #0
            lda (Anim),y
            bpl NormalFrame
 
            asl
            tay

            lda AnimVec,y
            sta Temp
            lda AnimVec+1,y
            sta Temp+1
            jmp (Temp)

    ;------------------------------------------------------------------------------
    ; Automatically define tokens and a vector table to token-processing code
    ; This is fairly neat, as the ordering of the entries in the table auto-generates
    ; the appropriate equates to access the table correctly!

TOK         SET 128
            MAC TOKEN
TOKEN_{1}   equ TOK
TOK         SET TOK + 1
            VECTOR Animate{1}
            ENDM

AnimVec     TOKEN JUMP
            TOKEN LOCK
            TOKEN MOVE
            TOKEN UNLOCK
            TOKEN DONE
            TOKEN FLIP
            TOKEN COLOUR
            TOKEN SYNCH


    ;------------------------------------------------------------------------------

AnimateDONE
			lda ObjFlag,x
			and #(~OBJFLAG_WAIT)&$FF
			sta ObjFlag,x
			
			lda #1
			jmp AnimUp



    ;------------------------------------------------------------------------------

AnimateFLIP

            lda ObjFacing,x
            eor #D7
            sta ObjFacing,x

			lda #1
			jmp AnimUp


NormalFrame
         

            sta ObjFrame,x

            iny
            lda (Anim),y
            sta ObjDelay,x                  ; should be < 128

            clc
            lda Anim
            adc #2
            sta ObjAnimLO,x
            lda Anim+1
            adc #0
            sta ObjAnimHI,x
            rts

    ;------------------------------------------------------------------------------

AnimateLOCK

                ; lock other creature to this

            lda ObjXPosition,x
            pha
            lda ObjYPosition,x
            pha

            lda Flags
            ora #FLAG_LOCK
            sta Flags

            txa
            eor #1
            tax                         ; other player

            ldy #1
            lda (Anim),y
            tay
            jsr SetHandler              ; cause other player to react

            pla
            sta ObjYPosition,x
            pla
            sta ObjXPosition,x

            txa
            eor #1
            tax

            lda #2

AnimUp      clc
            adc Anim
            sta Anim
            lda Anim+1
            adc #0
            sta Anim+1

            jmp ReReAnim

    ;------------------------------------------------------------------------------

AnimateUNLOCK

            lda Flags
            and #(~FLAG_LOCK)&$FF
            sta Flags

            lda #1
            jmp AnimUp


    ;------------------------------------------------------------------------------


AnimateJUMP

                ; Handle a JUMP

            ldy #1
            lda (Anim),y
            sta ObjAnimLO,x
            iny
            lda (Anim),y
            sta ObjAnimHI,x
            jmp ReAnimate


AnimateCOLOUR
            ldy #1
            lda (Anim),y
            sta BGPlayer
			lda #2
			jmp AnimUp

    ;------------------------------------------------------------------------------
            
AnimateMOVE

            ldy #1

            lda (Anim),y
            sta Temp
            iny
            lda (Anim),y
            sta Temp+1

            lda ObjFacing,x
            bpl MoveRight

            sec
            lda #0
            sbc Temp
            sta Temp
            lda #0
            sbc Temp+1
            sta Temp+1

MoveRight

            clc
            lda DrawX
            adc Temp
            sta DrawX
            lda DrawX+1
            adc Temp+1
            sta DrawX+1

            iny
            lda (Anim),y
            clc
            adc DrawY
            sta DrawY

            clc
            lda Anim
            adc #4
            sta Anim
            lda Anim+1
            adc #0
            sta Anim+1

            jmp ReReAnim

AnimateSYNCH

#if BIG_SPRITES = 1
            ;lda #0
#else
            ;lda #2
#endif

            ;sta Phase
            lda #1
            jmp AnimUp

    ;------------------------------------------------------------------------------


    ; Animations processing code proceeds until it has a frame and duration
    ; The duration is in TV-frames, and counts down.  When 0, the next 'command' in the
    ; animation sequence is processed.  Animations must use the GOTO to halt or repeat
    ; Animations are tokenised interpreted little programs.

    ; Commands/Macros
    ;   GOTO label              Vector to the label
    ;   LOCK event              Lock opponents together, sending the opponent the event
    ;   MOVE x,y                Adjust position by x,y pixels
    ;   UNLOCK                  Unlock opponents
    ;   DONE                    Indicate animation finished (clears OBJFLAG_WAIT for creature)
    ;   FLIP                    Mirror creature
    ;   SHOW frame,delay        Set th creature rame and delay (also halts animation till delay counts to 0)



ANIMATION_STAND


;        SYNCH



ANIMATION_STAND2

        SHOWRGB 2,4
        SHOWRGB 3,4
        SHOWRGB 6,4
        SHOWRGB 7,4
        SHOWRGB 8,4
        SHOWRGB 10,4
        SHOWRGB 11,4

        SHOWRGB 2,4
        SHOWRGB 3,4
        SHOWRGB 6,4
        SHOWRGB 7,4
        SHOWRGB 8,4
        SHOWRGB 10,4
        SHOWRGB 11,4


;        SHOWRGB KAPOW,30

        GOTO ANIMATION_STAND2


ANIMATION_RIGHT

        ;SHOWRGB 11,10
        GOTO ANIMATION_STAND2




; OBJECT Animation END
;------------------------------------------------------------------------------
 

;------------------------------------------------------------------------------
; STATE Processing
; *MUST* BE IN ONE BANK

    MAC FACEOPPONENT

            txa
            eor #1
            tay
            lda ObjFacing,y
            sta ObjFacing,x
    
    ENDM

	MAC WAIT
	SUBROUTINE
.HERE
			lda #<.HERE
			sta ObjFSMLO,x
			lda #>.HERE
			sta ObjFSMHI,x

		; wait for animation to complete

			lda ObjFlag,x
			and #OBJFLAG_WAIT
			bne .CONT
			
		    lda #<{1}
            sta Temp
            lda #>{1}
            sta Temp+1
            jmp (Temp)						; vector to the handler now the wait has completed
.CONT
		; Or, wait is incomplete, so fall through

	ENDM



SetState

    ; Set object's state base
    ; x = obj
    ; y = state table (*2)

            lda StateTables,y
            sta ObjStateLO,x
            lda StateTables+1,y
            sta ObjStateHI,x

            ldy #EVENT_DEFAULT*2                ; fall through and include DEFAULT init for handler

SetHandler

    ; Change object's processing code
    ; x = obj
    ; y = new handler entry (word access)

            lda ObjStateHI,x
            beq NullHandler                     ; don't handle anything with no entry

            sta Temp+1
            lda ObjStateLO,x
            sta Temp

            iny
            lda (Temp),y
            beq NullHandler                     ; no handler present for given event
            sta ObjFSMHI,x
            dey
            lda (Temp),y
            sta ObjFSMLO,x

   

ProcessFSM

    ; Vector to the creature's processing code
    ; x = creature

            lda ObjFSMLO,x
            sta Temp
            lda ObjFSMHI,x
            sta Temp+1
            jmp (Temp)

NullHandler rts




    ;------------------------------------------------------------------------------
    ; List of events

EVENT_DEFAULT           = 0                     ; default entry when switching states
EVENT_JOY_NULL          = 1
EVENT_JOY_UP            = 2
EVENT_JOY_DOWN          = 3
EVENT_JOY_LEFT          = 4
EVENT_JOY_RIGHT         = 5
EVENT_ATTACK_THROW      = 6

EVENT_INIT2             = 2


    ;------------------------------------------------------------------------------
    ; Automatically define states and a vector table to state-processing code
    ; This is fairly neat, as the ordering of the entries in the table auto-generates
    ; the appropriate equates to access the table correctly!

STATEQ      SET 0
            MAC STATENTRY
STATE_{1}   equ STATEQ
STATEQ      SET STATEQ + 1
            VECTOR State{1}
            ENDM

StateTables STATENTRY NORMAL
            STATENTRY WAIT

    ;------------------------------------------------------------------------------
    ; NORMAL state table
    ; Each State table is a table of pointers to code processing 'EVENTS'

StateNORMAL
            VECTOR NormalInit                ; 0     EVENT_DEFAULT
            VECTOR NormalWait                ; 1     EVENT_JOY_NULL
            VECTOR NormalInitRight ;0 ;GetUp                     ; 4     EVENT_JOY_UP
            VECTOR 0 ;NormalInit;AttackThrow1              ; 5     EVENT_JOY_DOWN
            VECTOR 0;WalkTurnAround            ; 6     EVENT_JOY_LEFT
            VECTOR 0;WalkStart                 ; 7     EVEMT_JOY_RIGHT
            VECTOR 0;React_AttackThrow         ; 8     EVENT_ATTACK_THROW

    ;------------------------------------------------------------------------------


StateWAIT

        ; allows moves to complete without joystick interaction 
        ; TODO: should still react to other creature moves

			VECTOR BusyInit                  ; 0     EVENT_DEFAULT
            VECTOR 0                         ; 1     EVENT_JOY_NULL
            VECTOR 0                         ; 4     EVENT_JOY_UP
            VECTOR 0					     ; 5     EVENT_JOY_DOWN
            VECTOR 0						 ; 6     EVENT_JOY_LEFT
            VECTOR 0						 ; 7     EVEMT_JOY_RIGHT
            VECTOR 0                         ; 8     EVENT_ATTACK_THROW

    ;------------------------------------------------------------------------------
    ; BUSY
    ; Waits until the animation clears the wait flag
    ; Then reverts to STATE_NORMAL

BusyInit	WAIT BusyDone
			rts

BusyDone	STATE NORMAL


    ;------------------------------------------------------------------------------
    ; NORMAL
    ; The basic default standing, doing nothing.


NormalInit
            ANIMATION ANIMATION_STAND
            HANDLER EVENT_JOY_NULL
            rts

NormalWait
           ;WAIT NormalInit
            rts


NormalInitRight

            ANIMATION ANIMATION_STAND ;RIGHT
            HANDLER EVENT_JOY_NULL
            rts



; STATE Processing END
;------------------------------------------------------------------------------


            SEG FixedBank
            ORG $7F800
            RORG $f800

VBProcessing

      
            ldx Player


            jsr ProcessFSM                  ; process creature FSM logic

            lda ObjXPosition,x
            sta DrawX
            lda #1
            sta DrawX+1
            lda ObjYPosition,x
            sta DrawY

            SETBANK INITBANK

            jsr AnimateObject               ; process object animation sequence
            jsr CalculateDrawPosition       ; setup position(s) for next VB draw


            ldy Player

            lda ObjFacing,y
            and #D7
            lsr
            lsr
            lsr
            lsr
            sta REFP0
            sta REFP1                   ;@17

            lda BGColour
            sta COLUBK

_RTS14      nop
            rts                         ; 14-cycle waste

    ;------------------------------------------------------------------------

NewFrame



            inc FrameCount
            bne nF2
            jmp Reset
nF2


    ; Start of vertical blank processing

;            lda #$02
;            sta WSYNC

;            sta VBLANK          

;            sta VSYNC           
;            sta WSYNC           
;            sta WSYNC
;            sta WSYNC

;            lda #$00
;            sta VSYNC           

            VERTICAL_SYNC

            lda #43             
            sta TIM64T

            lda #$00
            sta HMCLR

            lda SWCHB
            lda #D7
            sta switch


            jsr VBProcessing        ; spend 2000+ cycles

#if DEBUG
            lda INTIM
            bne Whoops
            inc BGColour            ; Overflow
Whoops
#endif



VblankLoop  lda INTIM
            bne VblankLoop      
            
            sta WSYNC           
            sta VBLANK          

    ;------------------------------------------------------------------------------
    ; START OF DISPLAY


            lda #238                ; NTSC
            sta TIM64T


    ;?
            lda     #%00010100      ; reflect, priority, 2 pixel wide ball
            sta     CTRLPF
    ;^?

            lda #PFCOL
            sta COLUPF
            
            lda #0
            sta PF0
            sta PF1
            sta PF2


            sta GRP0
            sta GRP1

            lda BGPlayer
            sta COLUP0
            sta COLUP1                  ; tentative colour


            lda FrameBank
            sta BANK



    ;--------------------------------------------------------------------------
    ; Init the loop counters (16 rows each) for each of the rows

            lda #127
            sta LoopCount

            ldy Player
            lda ObjFrame,y
            bmi SkipDraw

  
            ldx ObjPhase,y
            lda NextPhase,x
            sta ObjPhase,y
            
;            cpy #0
;            bne PalCols

;            ldx switch
;            bpl NTSC2

;PalCols     clc
;            adc #3

NTSC2       asl
            tax
            lda PhasePtr,x
            sta RGBVec
            lda PhasePtr+1,x
            sta RGBVec+1

            jsr PositionObject
            jsr DrawSprite

            sta WSYNC
            
;            lda #RED
;            sta COLUBK

SkipDraw    lda #0
            sta GRP0
            sta GRP1
            sta GRP0            ; buffered??
            sta GRP1            ; buffered??

            sta VDELP0
            sta VDELP1

            sta REFP0
            sta REFP1



;-----------------


    ;--------------------------------------------------------------------------
    ; START OF COLOUR BITMAP DISPLAY KERNEL


;PadScreen1  lda INTIM
;            bne PadScreen1



;-------------------



;            lda #254
;            sta PF2

;            ldx #8
;x2          sta WSYNC
;            dex
;            bne x2

;            stx PF2
            sta WSYNC

PadScreen2  lda INTIM
            bne PadScreen2
  
            sta HMCLR
            sta WSYNC


;    lda switch
;    and #D7
;    beq NTSC

    lda #60
    sta TIM64T
PAL lda INTIM
    bne PAL

;NTSC


            lda #D6+D1
            sta VBLANK                      ; end of screen - enter blanking

OverscanStart

            lda #25 ;35             
            sta TIM64T


    ;--------------------------------------------------------------------------
    ; Switch to alternate player(s)
            
            dec Player
            bpl PlayerN
            lda #BIG_SPRITES-1
            sta Player
PlayerN


    ;--------------------------------------------------------------------------
    ; PLAYER INTERACTION


;            lda #0
;            sta SWACNT              ; all input bits (IMPLICITLY SET @ STARTUP)

;            lda SWCHA               ; joystick

;            ldx Player
;            bne P1shift

;            lsr
;            lsr
;            lsr
;            lsr

;P1shift     and #$f

;            ldy ObjFacing,x
;            bpl FacOK

;            tay
;            and #D3+D2
;            cmp #D3+D2
;            beq FacOK2

;            tya
;            eor #D3+D2                      ; swap left/right if mirrored
;FacOK       tay
;FacOK2

            SETBANK INITBANK

;            lda JoystickEvent,y
;           bmi NullJoy                     ; an unhandled joystick direction

;            asl
;            tay

            ldy #EVENT_JOY_NULL*2
            jsr SetHandler                  ; handle joystick event

;NullJoy
;                lda     SWCHB
;                and     #$01
;                bne     NoResetSwitch

Overscan    lda INTIM
            bne Overscan

            sta WSYNC           
            sta WSYNC           

            jmp NewFrame




    ;--------------------------------------------------------------------------

CalculateDrawPosition

    ; Given the current frame of the player, calculate a draw position such that
    ; the centerpoint is at the creature's (ObjXPosition,ObjYPosition) and that the
    ; left and right edges are then forced to be onscreen.


    ; Calculate the frame definition start address.  Each frame is a 32-byte
    ; definition block giving the bank containing the frame and a matrix of
    ; 16-sprite numbers making up the nx6 grid.

    ; FORMAT:
    ;    BYTE       centerpointx, centerpointy
    ;    BYTES...    0, 1, 2, 3, 4, 5   \
    ;                6, 7, 8, 9,10,11    |  n x 6 matrix of 16-sprite #s
    ;                ....                |
    ;               24,25,26,27,28,29   /

            SETBANK INITBANK

            ldy Player
  
            lda #0
            sta FrameAddress+1

            lda ObjFrame,y              ; animation frame (absolute)
            asl
            adc ObjFrame,y              ; Frametable is bank, AddressLO, AddressHI
            tax

            lda FrameTable+1,x
            sta FrameAddress
            lda FrameTable+2,x
            sta FrameAddress+1

            lda FrameTable,x
            sta FrameBank
            sta BANK

            ldx #0
            lda ObjFacing,y
            bpl nMirror
            ldx #BIG_ROWS*BIG_COLS
nMirror


    ; Now we have a pointer to the frame definition block (FrameAddress)
    ; Go through the entire BIG_ROWS*BIG_COLS entries and convert the matrix 16-sprite
    ; number into an absolute address pointing to the sprite data.  This will correspond
    ; nicely to the vars used for the actual draw.

            ldy #0
            sty CenterX+1
            lda (FrameAddress),y
            sta CenterX
            iny
            lda (FrameAddress),y
            sta CenterY

            stx Temp+1

Mirrorable  iny

            inc Temp+1
            ldx Temp+1
            lda BaseMirror-1,x
            tax


            tya
            pha

            lda (FrameAddress),y
            asl
            tay
            lda T128,y
            sta SPBASE,x
            lda T128+1,y
            sta SPBASE+1,x

            pla
            tay


            cpy #BIG_ROWS*BIG_COLS+1
            bcc Mirrorable


    ;--------------------------------------------------------------------------
    ; Centerpoint Adjustment based on frame

            ldy Player

            lda ObjFrame,y
            bmi SkipRepos

            lda ObjFacing,y
            bpl nM

            clc
            lda #0
            sbc CenterX
            sta CenterX
            lda #0
            sbc CenterX+1
            sta CenterX+1

            clc
            lda CenterX
            adc #48
            sta CenterX
            lda CenterX+1
            adc #0
            sta CenterX+1

nM

    ; Adjust draw position so that centerpoint is where the ObjXPosition is

            sec
            lda DrawX
            sbc CenterX
            sta VisualX
            lda DrawX+1
            sbc CenterX+1
            sta VisualX+1

            bne OKleft


    ; OK, so the object is offscreen (to the left)
    ; This means that, with the centerpoint adjustment, the left edge of the sprite is offscreen
    ; So we need to adjust the object position (DrawX,DrawY) such that the sprite will be onscreen
    ; As a minimum, we know we want the sprite to be at (0,DrawY).  So we just adjust the
    ; object position by the negative of the offsreen offset

            sec
            lda #0
            sbc VisualX                     ; all we really care about - the offset
            sta Temp
            adc DrawX                       ; adjust (what will be) the actual coordinate
            sta DrawX

    ; and set the visual draw position to left edge

;            lda #1
;            sta VisualX+1
            lda #0
            sta VisualX
            beq Onscreen

OKleft

    ; Check for right-hand side boundary conditions


            sec
            lda VisualX
            sbc #160-48
            sta Temp
            lda VisualX+1
            sbc #1
            bcc Onscreen

            lda Temp
            eor #$FF
            adc #0
            sta Temp

            adc DrawX
            sta DrawX


            lda #160-48
            sta VisualX
;            lda #1
;            sta VisualX+1

Onscreen

            sec
            lda DrawY
            sbc CenterY
            sta VisualY
SkipRepos

    ;--------------------------------------------------------------------------

            ldx Player
            lda DrawX
            sta ObjXPosition,x
            lda DrawY
            sta ObjYPosition,x


            rts


T128
.T128        SET $F000

    REPEAT 22
        .word .T128
.T128        SET .T128 + 128
    REPEND

    ;--------------------------------------------------------------------------


Reset


            SETBANK TITLESCREEN

            lda #$80
            
            sta Audio+1             ; audio pointer
            lda #0
            sta Audio

            lda #0
            sta PCount

rr

   ;         ldy PCount
   ;         lda PFrame,y
   ;         bpl PCOKC
   ;         lda #0
   ;         sta PCount
   ;         beq rr
PCOKC
   ;         ldy #3
   ;         jsr TitleScreen

   ;         inc PCount

   ;         clc
   ;         lda Audio
   ;         adc #0
   ;         sta Audio
   ;         lda Audio+1
   ;         adc #1
  ;          sta Audio+1

;;            inc Audio+1

;            jmp rr


    ;PFrame gives list of frames to play!

;PFrame      .byte 0,1,0,1,0,1,0,1,0,1,0,1,0,1,4,5,4,5,4,5,4,5,4,5,-1



            SETBANK INITBANK
            jmp Cart_Init                   ; init code is in bank 1!




;------------------------------------------------------------------------------

#if 0
VectorToBankedSubroutine

            lda RAM_BANK
            pha

            lda SubroutineBank,y
            sta RAM_BANK
            sta BANK

            lda SubroutineAddrLO,y
            sta temp
            lda SubroutineAddrHI,y
            sta temp+1

            jsr ReVector

            pla
            sta RAM_BANK
            sta BANK

            rts

ReVector    jmp (temp)


SubroutineBANK
#endif




;------------------------------------------------------------------------------


JoystickEvent

        .byte $FF,$FF,$FF,$FF,$FF,$FF,$FF,EVENT_JOY_RIGHT,$FF,$FF,$FF,EVENT_JOY_LEFT,$FF,EVENT_JOY_DOWN,EVENT_JOY_UP,EVENT_JOY_NULL


    ;--------------------------------------------------------------------------
    ; Object X,Y positioning
    ; Timing is absolutely critical here!

PositionObject

    ; Waste scanlines to start of object position, giving our vertical movement

            ldx VisualY
            inx
DelayVert   sta WSYNC
            dex
            bne DelayVert

    ; Now the horizontal magic (which isn't working (1/MAR))

            ldx VisualX
            clc
            lda MoveTable,x
            sta HMP0
            adc #$10
            sta HMP1

            lda #1
            sta VDELP0
            sta VDELP1

            lda #3
            sta NUSIZ0
            sta NUSIZ1

            sta WSYNC

            lda #>JNDelayPos
            sta Temp+1
            lda DelayTable,x
            sta Temp
            jmp (Temp)

            ALIGN   256

JNDelayPos  
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c5

            nop

            sta RESP0
            sta RESP1
            sta WSYNC
            sta HMOVE

            lda #>JNDelayDraw
            sta Temp+1
            lda #<JNDelayDraw
            clc
            adc DelayTable,x
            sta Temp

            ;lda #0
            ;sta PF2

            sta WSYNC

            SLEEP 20

            jmp (Temp)


JNDelayDraw

            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c5

            rts


            ALIGN 256
DelayTable
            .byte   37, 37
Q           SET     36
            REPEAT  37
            .byte   Q, Q, Q
Q           SET     Q-1
            REPEND

MoveTable
           .byte   $30, $20
            REPEAT  37
                .byte   $40, $30, $20
            REPEND


;--------------------------------------------------------------------------
; The beautiful, beautiful, draw loop which draws the entire matrix onscreen
; This caters for an adjustable (at compile-time) number of rows and also
; facilitates double-size pixels.

            SUBROUTINE

.S1         SET SPBASE
.S2         SET SPBASE + 2
.S3         SET SPBASE + 4
.S4         SET SPBASE + 6
.S5         SET SPBASE + 8
.S6         SET SPBASE + 10

DrawSprite

;RED         = %00110100 ;$42        ; secam 4
;GREEN       = %11000110 ;C8 ;$D6              ; secam 8
;BLUE        = %01110010             ; secam 2
RED         = $4E ;%00110100 ;$42        ; secam 4
GREEN       = $CE ;%11000110 ;C8 ;$D6              ; secam 8
BLUE        = $8E ;%01110010             ; secam 2


            jmp (RGBVec)

NextPhase   .byte 2,0,0 ;1,2,0                     ; FIXED BANK ALWAYS
PhasePtr    .word .LABR,.LABG,.LABB
            .word .LABPALR,.LABPALG,.LABPALB

            ALIGN 256

.LABG       lda #GREEN          ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABB           ; 3
    
            bmi EndDraw

.LABR       lda #RED            ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABG           ; 3

            bmi EndDraw

.LABB       lda #BLUE           ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABR           ; 3

EndDraw     rts


            ALIGN 256

;PALRED         = $64
;PALGREEN       = $54
;PALBLUE        = $D4

PALRED         = $C8 ;$D6              ; secam 8
PALGREEN       = %00110100 ;$42        ; secam 4
PALBLUE        = %01110100             ; secam 2

.LABPALG    lda #PALGREEN       ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABPALB        ; 3
    
            bmi EndPalDraw

.LABPALR    lda #PALRED         ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABPALG        ; 3

            bmi EndPalDraw

.LABPALB    lda #PALBLUE        ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABPALR        ; 3

EndPalDraw  rts

 

BaseMirror

.SP         SET 0
            REPEAT BIG_ROWS*BIG_COLS
            .byte .SP
.SP         SET .SP + 2
            REPEND
.SPBASE     SET 0
            REPEAT BIG_ROWS
.SPOFF      SET 10
            REPEAT BIG_COLS
            .byte .SPBASE + .SPOFF
.SPOFF      SET .SPOFF-2
            REPEND
.SPBASE     SET .SPBASE + 12
            REPEND


;===============================================================================
; Fixed bank portion of Interleaved ChronoColour Draw System


    MAC CHRONOLINE ;{line}

        ;7+

            lda ChronoBank + {1}                ; 3
            sta BANK                            ; 3     = 6

            ;sta WSYNC                           ; 3

            lda ChronoColour + {1}              ; 3
            sta COLUPF                          ; 3     = 6

            lda ( LineR0 +  0 + {1} * 14 ),y    ; 5
            sta PF0                             ; 3
            lda ( LineR0 +  2 + {1} * 14 ),y    ; 5
            sta PF1                             ; 3
            lda ( LineR0 +  4 + {1} * 14 ),y    ; 5
            sta PF2                             ; 3     = 24

            lda ( LineR0 +  6 + {1} * 14 ),y    ; 5
            sta PF0                             ; 3
            lda ( LineR0 +  8 + {1} * 14 ),y    ; 5
            sta PF1                             ; 3
            lda ( LineR0 + 10 + {1} * 14 ),y    ; 5
            sta PF2                             ; 3     = 24

            lda (LineR0 + 12 + {1} * 14 ),y     ; 5
;            sta.w AUDV0                         ; 3     = 8

            lda.w 0

;            SLEEP 9

    ENDM

_rts        rts
sleep61     SLEEP (61-12)
            rts


    ALIGN 256

ChronoDraw

            lda RAM_BANK
            pha                     ; save callee's bank

            ldy #169                ; total number of scanlines to display

            sta WSYNC
 ;           jsr _rts
 ;           jsr _rts
 ;           jsr _rts
 ;           jsr _rts

            ldx Phase
            beq ScanRedStart        ; 2(3) --> 3 if taken
            dex                     ; 2
            beq ScanGreenStart      ; 2(3) --> 7 if taken
            bne ScanBlueStart       ; (3) --> 9

ScanRedStart
            ; @3

            SLEEP 6 

            ; now 9 cycles after WSYNC
            jsr sleep61

ScanRed     CHRONOLINE 0
            dey
            beq ScanEnd
            jmp ScanGreen

ScanGreenStart
            ; @7
            SLEEP 2
            jsr sleep61

ScanGreen   CHRONOLINE 1
            dey
            beq ScanEnd
            jmp ScanBlue

ScanBlueStart
                ; @9
            jsr sleep61


ScanBlue    CHRONOLINE 2
            dey
            beq ScanEnd
            jmp ScanRed


ScanEnd     pla
            sta RAM_BANK
            sta BANK                    ; restore previous bank for callee


            rts



    ;==========================================================================
    ; The reset vectors
    ; these must live in the fixed bank (last 2K of any ROM image in TigerVision)

            SEG InterruptVectors
            ORG $7FFFA
            RORG $7ffa

            VECTOR Reset           ; NMI        (not used)
            VECTOR Reset           ; RESET
            VECTOR Reset           ; IRQ        (not used)

    ;==========================================================================





            END
